const eol = require('os').EOL;
const path = require('path');
const paths = require('./paths');
const coverage = !!process.env.E2E_COVERAGE;
const {processStackTraceLine, splitStackTrace, internalStackTrace} = require('./protractor.util');

require('ts-node').register({
  project: 'e2e/tsconfig.json',
  compiler: 'typescript',
  transpileOnly: true,
});
require('@babel/polyfill');
require('@babel/register')({
  ignore: [/\/node_modules\//],
  cache: false,
});

module.exports.config = {
  seleniumAddress: process.env.E2E_CHROMEDRIVER_URL || 'http://localhost:9511',
  baseUrl: process.env.E2E_APPLICATION_URL || 'http://localhost:3000',

  getPageTimeout: parseInt(process.env.E2E_PAGE_TIMEOUT) || 10000,
  allScriptsTimeout: parseInt(process.env.E2E_ALL_SCRIPTS_TIMEOUT) || 15000,

  framework: 'jasmine',
  specs: [
    path.join(paths.appPath, 'e2e/**/*.spec.ts'),
    path.join(paths.appPath, 'e2e/**/*.spec.js'),
  ],

  SELENIUM_PROMISE_MANAGER: false,
  skipSourceMapSupport: true,

  plugins: [
    {
      package: 'protractor-istanbul-plugin',
      enabled: coverage,
      outputPath: path.join(paths.appPath, 'coverage', 'e2e-raw'),
    },
  ],

  params: {
    fileDir: process.env.E2E_FILES_DIR || path.join(paths.appPath, 'e2e', 'files'),
  },

  capabilities: {
    browserName: 'chrome',

    chromeOptions: {
      detach: true,
      args: [
        '--lang=de',
        '--window-size=1440,900',
        '--incognito',
        '--disable-extensions',
        ...(process.env.E2E_CHROME_FLAGS || '')
          .trim()
          .split(/\s+/)
          .filter(Boolean),
      ],
      prefs: {
        download: {
          default_directory: path.join(paths.appPath, 'e2e', 'tmp'),
        },
      },
    },
  },
  onPrepare: () => {
    browser.ignoreSynchronization = true;
    Error.stackTraceLimit = Infinity;

    // @see https://stackoverflow.com/a/33784794/11065462
    // @see https://github.com/angular/protractor/issues/5166
    const { Logger } = require('protractor/built/logger');
    Logger.prototype.warn = (message) => { throw message };

    const SpecReporter = require('jasmine-spec-reporter').SpecReporter;
    const JUnitXmlReporter = require('jasmine-reporters').JUnitXmlReporter;

    jasmine.getEnv().addReporter(new JUnitXmlReporter({
      savePath: path.join(paths.appPath, 'coverage'),
      filePrefix: 'junit-e2e',
      consolidateAll: true,
    }));
    jasmine.getEnv().addReporter(new SpecReporter({
      spec: {
        displayStacktrace: true,
        displayDuration: true,
      },
      suite: {
        displayNumber: true,
      },
      summary: {
        displayStacktrace: true,
        displayDuration: true,
      },
      colors: {
        enabled: true,
      },
      stacktrace: {
        filter: (stacktrace) => splitStackTrace(stacktrace)
          .map((line) => processStackTraceLine(line))
          .filter((line) => line)
          .join(eol) || internalStackTrace,
      },
    }));
  },
  jasmineNodeOpts: {
    defaultTimeoutInterval: parseInt(process.env.E2E_JASMINE_TIMEOUT) || 30000,
    print: () => {
    },
  },
};
