const { sep } = require('path');
const { getOptions } = require('loader-utils');
const escapeRegexp = require('escape-string-regexp');

function updateSourceMap(sourceMap, options) {
  const sourceRoot = options.sourceRoot || __dirname;
  const extensions = options.extensions.map(escapeRegexp) || ['\\.css'];
  const pathRegExp = new RegExp(`^${escapeRegexp(sourceRoot)}${escapeRegexp(sep)}(.+(?:${extensions.join('|')}))$`);
  return Object.assign({}, sourceMap, {
    sources: sourceMap.sources.map((path) => path.replace(pathRegExp, 'webpack:///$1'))
  });
}

// @see https://github.com/bholloway/adjust-sourcemap-loader#how-it-works
module.exports = function(source, map) {
  const options = getOptions(this);
  if (!options.enabled) {
    this.callback(null, source, map);
    return;
  }

  // format:
  // ...
  // exports.push([
  //      module.id,
  //      "CSS_TEXT_HERE",
  //      "",
  //      {"version":3,"sources":["/full/path.css"],"names":[],"mappings":"AAAA;BBBB","file":"path.css","sourcesContent":["SOME_CONTENT"],"sourceRoot":""}
  // ]);
  // ...
  const sourceMapStart = source.indexOf('{"version":3,');
  const sourceMapEnd = source.lastIndexOf('"sourceRoot":""}');
  if (sourceMapStart < 0 || sourceMapEnd < 0) {
    this.callback(null, source, map);
    return;
  }

  try {
    const encodedSourceMap = source.substring(sourceMapStart, sourceMapEnd + '"sourceRoot":""}'.length);
    const decodedSourceMap = JSON.parse(encodedSourceMap);
    const updatedSourceMap = updateSourceMap(decodedSourceMap, options);

    const newSource = (
      source.substring(0, sourceMapStart) +
      JSON.stringify(updatedSourceMap) +
      source.substring(sourceMapEnd + '"sourceRoot":""}'.length)
    );
    this.callback(null, newSource, map);
  } catch (e) {
    this.callback(null, source, map);
  }
};
